@@ -21,7 +21,7 @@ from flare.predict import (
 )
 from flare.utils.element_coder import NumpyEncoder
 
-from ase.calculators.calculator import Calculator
+from ase.calculators.calculator import Calculator, all_changes
 
 
 def get_rebuild_from_err(err_msg, rebuild_dict, newbound_dict):
@@ -58,6 +58,8 @@ class FLARE_Calculator(Calculator):
             by default.
     """
 
+    implemented_properties = ["energy", "forces", "stress", "stds"]
+
     def __init__(
         self, gp_model, mgp_model=None, par=False, use_mapping=False, **kwargs
     ):
@@ -68,26 +70,10 @@ class FLARE_Calculator(Calculator):
         self.par = par
         self.results = {}
 
-    def get_property(self, name, atoms=None, allow_calculation=True):
-        if name not in self.results.keys():
-            if not allow_calculation:
-                return None
-            self.calculate(atoms)
-        return self.results[name]
-
-    def get_potential_energy(self, atoms=None, force_consistent=False):
-        return self.get_property("energy", atoms)
-
-    def get_forces(self, atoms):
-        return self.get_property("forces", atoms)
-
-    def get_stress(self, atoms):
-        return self.get_property("stress", atoms)
-
     def get_uncertainties(self, atoms):
         return self.get_property("stds", atoms)
 
-    def calculate(self, atoms):
+    def calculate(self, atoms=None, properties=None, system_changes=all_changes):
         """
         Calculate properties including: energy, local energies, forces,
             stress, uncertainties.
@@ -96,6 +82,13 @@ class FLARE_Calculator(Calculator):
             atoms (FLARE_Atoms): FLARE_Atoms object
         """
 
+        super().calculate(
+            atoms=atoms, properties=properties, system_changes=system_changes
+        )
+
+        if properties is None:
+            properties = self.implemented_properties
+
         if self.use_mapping:
             self.calculate_mgp(atoms)
         else:
@@ -119,26 +112,12 @@ class FLARE_Calculator(Calculator):
             )
 
         # Set the energy, force, and stress attributes of the calculator.
-        res_name = [
-            "local_energies",
-            "forces",
-            "partial_stresses",
-            "local_energy_stds",
-            "stds",
-            "partial_stress_stds",
-        ]
-        res_dims = [1, 3, 6, 1, 3, 6]
-        for i in range(len(res_name)):
-            if len(res[i].shape) == 2:
-                assert (
-                    res[i].shape[1] == res_dims[i],
-                    f"{res_name[i]} shape doesn't match, "
-                    f"{res[i].shape[1]} and {res_dims[i]}",
-                )
-            elif len(res[i].shape) == 1:
-                assert res_dims[i] == 1
-
-            self.results[res_name[i]] = res[i]
+        self.results["local_energies"] = res[0]
+        self.results["forces"] = res[1]
+        self.results["partial_stresses"] = -res[2][:, [0, 3, 5, 4, 2, 1]]
+        self.results["local_energy_stds"] = res[3]
+        self.results["stds"] = res[4]
+        self.results["partial_stress_stds"] = res[5][:, [0, 3, 5, 4, 2, 1]]
 
     def calculate_mgp(self, atoms):
         nat = len(atoms)
@@ -161,7 +140,7 @@ class FLARE_Calculator(Calculator):
                 f, v, vir, e = self.mgp_model.predict(chemenv)
                 self.results["forces"][n] = f
                 self.results["partial_stresses"][n] = vir
-                self.results["stds"][n] = np.sqrt(np.absolute(v))
+                self.results["stds"][n][0] = np.sqrt(np.absolute(v))
                 self.results["local_energies"][n] = e
 
             except ValueError as err_msg:  # if lower_bound error is raised
@@ -185,21 +164,18 @@ class FLARE_Calculator(Calculator):
                 n, chemenv = ra
                 f, v, vir, e = self.mgp_model.predict(chemenv)
                 self.results["forces"][n] = f
-                self.results["partial_stresses"][n] = vir
-                self.results["stds"][n] = np.sqrt(np.absolute(v))
+                self.results["partial_stresses"][n] = -vir[[0, 3, 5, 4, 2, 1]]
+                self.results["stds"][n][0] = np.sqrt(np.absolute(v))
                 self.results["local_energies"][n] = e
 
     def calculation_required(self, atoms, quantities):
         return True
 
-    def reset(self):
-        """Clear the results attribute of the calculator."""
-
-        self.results = {}
-
     def as_dict(self):
         outdict = {}
 
+        outdict["class"] = self.__class__.__name__
+
         gp_dict = self.gp_model.as_dict()
         outdict["gp_model"] = gp_dict
 
@@ -247,3 +223,6 @@ class FLARE_Calculator(Calculator):
             calc = FLARE_Calculator.from_dict(json.loads(f.readline()))
 
         return calc
+
+    def build_map(self):
+        self.mgp_model.build_map(self.gp_model)
