@@ -1,7 +1,7 @@
 """
 This module is to provide the same interface as the module `dft_interface`, so we can use ASE atoms and calculators to run OTF
 """
-
+import os
 import numpy as np
 from copy import deepcopy
 
@@ -21,7 +21,7 @@ def parse_dft_input(atoms):
     return pos, spc, cell, mass_dict
 
 
-def run_dft_par(atoms, structure, dft_calc, **dft_kwargs):
+def run_dft_par(atoms, structure, dft_calc, dft_kwargs, **kwargs):
     """
     Assume that the atoms have been updated
     """
@@ -29,6 +29,14 @@ def run_dft_par(atoms, structure, dft_calc, **dft_kwargs):
     calc = deepcopy(dft_calc)
     atoms.set_calculator(calc)
 
+    # clean up previous results
+    if dft_kwargs is not None and dft_kwargs.get("cleanup", None):
+        if "dft" in os.listdir():
+            if "WAVECAR" in os.listdir("dft"):
+                os.remove("dft/WAVECAR")
+        if "WAVECAR" in os.listdir():
+            os.remove("WAVECAR")
+
     # Calculate DFT energy, forces, and stress.
     # Source code for DFT parser:
     # https://wiki.fysik.dtu.dk/ase/_modules/ase/io/espresso.html#read_espresso_out
@@ -39,4 +47,6 @@ def run_dft_par(atoms, structure, dft_calc, **dft_kwargs):
     # The results will be written to the 'results' dictionary, and structure.forces
     # directly return the atoms.calc.results['forces']
 
+    structure.calc.results = atoms.calc.results
+
     return forces
