@@ -40,15 +40,15 @@ class AtomicEnvironment:
     bonds, triples, and many body interaction. This dictionary should be
     consistent with the hyps_mask used in the GuassianProcess object.
 
-    * specie_mask: 118-long integer array descirbing which elements belong to
+    * species_mask: 118-long integer array descirbing which elements belong to
         like groups for determining which bond hyperparameters to use.
         For instance, [0,0,1,1,0 ...] assigns H to group 0, He and
         Li to group 1, and Be to group 0 (the 0th register is ignored).
     * nspecie: Integer, number of different species groups (equal to number of
-        unique values in specie_mask).
+        unique values in species_mask).
     * ntwobody: Integer, number of different hyperparameter/cutoff sets to
         associate with different 2-body pairings of atoms in groups defined in
-        specie_mask.
+        species_mask.
     * twobody_mask: Array of length nspecie^2, which describes the cutoff to
         associate with different pairings of species types. For example, if
         there are atoms of type 0 and 1, then twobody_mask defines which cutoff
@@ -59,12 +59,12 @@ class AtomicEnvironment:
         used for different types of bonds defined in twobody_mask
     * ncut3b: Integer, number of different cutoffs sets to associate
         with different 3-body pariings of atoms in groups defined in
-        specie_mask.
+        species_mask.
     * cut3b_mask: Array of length nspecie^2, which describes the cutoff to
         associate with different bond types in triplets. For example, in a
         triplet (C, O, H) , there are three cutoffs. Cutoffs for CH bond, CO
         bond and OH bond. If C and O are associate with atom group 1 in
-        specie_mask and H are associate with group 0 in specie_mask, the
+        species_mask and H are associate with group 0 in species_mask, the
         cut3b_mask[1*nspecie+0] determines the C/O-H bond cutoff, and
         cut3b_mask[1*nspecie+1] determines the C-O bond cutoff. If we want the
         former one to use the 1st cutoff in threebody_cutoff_list and the later
@@ -123,7 +123,7 @@ class AtomicEnvironment:
         self.nmanybody = 0
 
         self.nspecie = 1
-        self.specie_mask = None
+        self.species_mask = None
         self.twobody_mask = None
         self.threebody_mask = None
         self.manybody_mask = None
@@ -173,8 +173,8 @@ class AtomicEnvironment:
             self.cutoffs["twobody"] = self.twobody_cutoff
 
         self.nspecie = cutoffs_mask.get("nspecie", 1)
-        if "specie_mask" in cutoffs_mask:
-            self.specie_mask = np.array(cutoffs_mask["specie_mask"], dtype=np.int)
+        if "species_mask" in cutoffs_mask:
+            self.species_mask = np.array(cutoffs_mask["species_mask"], dtype=int)
 
         for kernel in AtomicEnvironment.all_kernel_types:
             ndim = AtomicEnvironment.ndim[kernel]
@@ -211,7 +211,7 @@ class AtomicEnvironment:
                 self.species,
                 self.sweep_array,
                 self.nspecie,
-                self.specie_mask,
+                self.species_mask,
                 self.twobody_mask,
             )
 
@@ -234,7 +234,7 @@ class AtomicEnvironment:
                 self.threebody_cutoff,
                 self.threebody_cutoff_list,
                 self.nspecie,
-                self.specie_mask,
+                self.species_mask,
                 self.cut3b_mask,
             )
 
@@ -261,7 +261,7 @@ class AtomicEnvironment:
                 self.species,
                 self.sweep_array,
                 self.nspecie,
-                self.specie_mask,
+                self.species_mask,
                 self.manybody_mask,
                 cf.quadratic_cutoff,
             )
