@@ -6,6 +6,8 @@ or running an MD simulation updated on-the-fly.
 import datetime
 import logging
 import time
+import sys
+import copy
 import numpy as np
 
 from logging import FileHandler, StreamHandler, Logger
@@ -13,6 +15,7 @@ from os.path import isfile
 from shutil import move as movefile
 from typing import Union, List
 
+import flare
 from flare.struc import Structure
 from flare.utils.element_coder import Z_to_element
 
@@ -134,6 +137,75 @@ class Output:
         f = logging.getLogger(self.basename + "log")
         f.info(f"{datetime.datetime.now()}")
 
+        # Write package version numbers
+        f.info(f"python {sys.version}")
+        f.info(f"numpy {np.__version__}")
+        try:
+            import ase
+
+            f.info(f"ase {ase.__version__}")
+        except:
+            pass
+
+        try:
+            import pymatgen
+
+            f.info(f"pymatgen {pymatgen.__version__}")
+        except:
+            pass
+
+        try:
+            f.info(f"flare {flare.__version__}")
+        except AttributeError:  # using customized package not from pip install
+            dirs = flare.__path__[0].split("/")
+            dirs_rep = copy.copy(dirs)
+            dirs_rep[-1] = "setup.py"
+            try:
+                setup_path = "/".join(dirs_rep)
+                setup_py = open(setup_path)
+            # In the Github Action test of the flare++ tutorial,
+            # flare.__version__ throws an AttributeError, but there isn't a flare/flare subdirectory. So we need another exception block.
+            except FileNotFoundError:
+                dirs_add = dirs + ["setup.py"]
+                setup_path = "/".join(dirs_add)
+                setup_py = open(setup_path)
+
+            for line in setup_py.readlines():
+                line = line.strip()
+                if "version=" in line:
+                    f.info(f"flare {line[9:len(line)-2]}")
+                    break
+            setup_py.close()
+
+        try:
+            import flare_pp
+
+            f.info(f"flare_pp {flare_pp.__version__}")
+        except AttributeError:
+            import flare_pp
+
+            dirs = flare_pp.__path__[0].split("/")
+            dirs_rep = copy.copy(dirs)
+            dirs_rep[-1] = "setup.py"
+            try:
+                setup_path = "/".join(dirs_rep)
+                setup_py = open(setup_path)
+            except FileNotFoundError:
+                dirs_add = dirs + ["setup.py"]
+                setup_path = "/".join(dirs_add)
+                setup_py = open(setup_path)
+
+            for line in setup_py.readlines():
+                line = line.strip()
+                if "version=" in line:
+                    f.info(f"flare_pp {line[9:len(line)-2]}")
+                    break
+            setup_py.close()
+
+        except ModuleNotFoundError:
+            pass
+
+        # Write uncertainty tolerance
         if isinstance(std_tolerance, tuple):
             std_string = (
                 "Relative uncertainty tolerance: "
@@ -330,6 +402,7 @@ class Output:
         dft_forces: np.array = None,
         dft_energy=0,
         predict_energy=float("nan"),
+        target_atoms=None,
     ):
         """write atomic configuration in xyz file
 
@@ -354,6 +427,7 @@ class Output:
             timestep=curr_step,
             write_file="",
             append=False,
+            target_atoms=target_atoms,
         )
 
         logger = logging.getLogger(self.basename + "xyz")
@@ -385,7 +459,7 @@ class Output:
         else:
             for i, hyp in enumerate(hyps):
                 f.info(f"Hyp{i} : {hyp:.4f}")
-        if isinstance(like,list):
+        if isinstance(like, list):
             f.info(f"Likelihood: {np.round(like,4)}")
         else:
             f.info(f"Likelihood: {like:.4f}")
