@@ -255,25 +255,13 @@ class RobustBayesianCommitteeMachine(GaussianProcess):
 
         self.training_data += [[]]
         self.training_labels += [[]]
-        self.training_labels_np += [
-            np.empty(
-                0,
-            )
-        ]
+        self.training_labels_np += [np.empty(0)]
         self.n_envs_prev += [0]
 
         self.training_structures += [[]]  # Environments of each structure
         self.energy_labels += [[]]  # Energies of training structures
-        self.energy_labels_np += [
-            np.empty(
-                0,
-            )
-        ]
-        self.all_labels += [
-            np.empty(
-                0,
-            )
-        ]
+        self.energy_labels_np += [np.empty(0)]
+        self.all_labels += [np.empty(0)]
 
         self.ky_mat += [None]
         self.force_block += [None]
@@ -780,7 +768,7 @@ class RobustBayesianCommitteeMachine(GaussianProcess):
             self.sync_experts_data(i)
 
     def unsync_experts_data(self, expert_id: int):
-        """ Reset global variables. """
+        """Reset global variables."""
         if len(self.training_data) > expert_id:
             _global_training_data.pop(f"{self.name}_{expert_id}", None)
             _global_training_labels.pop(f"{self.name}_{expert_id}", None)
@@ -788,7 +776,7 @@ class RobustBayesianCommitteeMachine(GaussianProcess):
             _global_energy_labels.pop(f"{self.name}_{expert_id}", None)
 
     def sync_experts_data(self, expert_id: int):
-        """ Reset global variables. """
+        """Reset global variables."""
         if len(self.training_data) > expert_id:
             _global_training_data[f"{self.name}_{expert_id}"] = self.training_data[
                 expert_id
@@ -854,7 +842,7 @@ class RobustBayesianCommitteeMachine(GaussianProcess):
         self.n_envs_prev[expert_id] = len(self.training_data[expert_id])
 
     def redistribute_training_data(self, figure_name="default"):
-        """ redistribute data """
+        """redistribute data"""
 
         joint_data = []
         joint_structures = []
@@ -922,7 +910,7 @@ class RobustBayesianCommitteeMachine(GaussianProcess):
         return kmat
 
     def compute_join_dist_mat(self, list_expert_id):
-        """ Reset global variables. """
+        """Reset global variables."""
         joint_data = []
         joint_structures = []
         for i in list_expert_id:
@@ -1082,7 +1070,6 @@ class RobustBayesianCommitteeMachine(GaussianProcess):
         thestr += f"Cutoffs: {self.cutoffs}\n"
         thestr += f"Number of Experts: {len(self.training_data)}\n"
 
-
         thestr += f"Number of hyperparameters: {len(self.hyps)}\n"
         thestr += f"Hyperparameters_array: {str(self.hyps)}\n"
         thestr += "Hyperparameters: \n"
