@@ -8,6 +8,7 @@ input coordinates don't need to lie inside the box.
 
 from typing import List, Union, Any
 from json import dumps, loads
+from abc import abstractmethod
 
 import pickle as pickle
 import numpy as np
@@ -328,17 +329,19 @@ class Structure:
         """
         struc = Structure(
             cell=np.array(dictionary.get("_cell", dictionary.get("cell"))),
+            species=dictionary["coded_species"],
             positions=np.array(
                 dictionary.get("_positions", dictionary.get("positions"))
             ),
-            species=dictionary["coded_species"],
-            forces=np.array(dictionary.get("forces")),
             mass_dict=dictionary.get("mass_dict"),
+            prev_positions=dictionary.get("prev_positions", None),
             species_labels=dictionary.get("species_labels"),
+            forces=np.array(dictionary.get("forces")),
+            stds=np.array(dictionary.get("stds")),
             energy=dictionary.get("energy", None),
         )
 
-        struc.stds = np.array(dictionary.get("stds"))
+        struc.stress = dictionary.get("stress", None)
 
         return struc
 
@@ -371,7 +374,7 @@ class Structure:
             energy = None
 
         try:
-            stress = atoms.get_stress()
+            stress = atoms.get_stress()  # TODO: what stress order should we use?
         except:
             stress = None
 
@@ -398,8 +401,7 @@ class Structure:
         properties = ["forces", "energy", "stress"]
         for p in properties:
             results[p] = getattr(self, p)
-        calculator = SinglePointCalculator(atoms, **results)
-        atoms.set_calculator(calculator)
+        atoms.calc = SinglePointCalculator(atoms, **results)
 
         return atoms
 
@@ -478,6 +480,7 @@ class Structure:
         predict_energy=None,
         dft_forces=None,
         dft_energy=None,
+        target_atoms=None,
         timestep=-1,
         write_file: str = "",
         append: bool = False,
@@ -512,6 +515,9 @@ class Structure:
                 xyz_str += f" PE={predict_energy}"
             if dft_energy is not None:
                 xyz_str += f" DFT_PE={dft_energy}"
+            if target_atoms is not None:
+                indices = " ".join([str(t) for t in target_atoms])
+                xyz_str += f' target_atoms="{indices}"'
             xyz_str += f' Proprties="species:S:1:pos:R:3'
 
             if print_stds:
@@ -567,7 +573,7 @@ class Structure:
 
     @staticmethod
     def from_file(
-        file_name: str, format: str = ""
+        file_name: str, format: str = "", as_trajectory: bool = False
     ) -> Union["flare.struc.Structure", List["flare.struc.Structure"]]:
         """
         Load a FLARE structure from a file or a series of FLARE structures
@@ -615,6 +621,118 @@ class Structure:
         elif is_poscar and not _pmg_present:
             raise ImportError("Pymatgen not imported; functionality requires pymatgen.")
 
+    def __eq__(self, other):
+        """
+        Very crude equality measure that will check to see if positions,
+        cell, and species are identical. Will not detect e.g. symmetry
+        equivalence, translation, if one structure is a supercell of another,
+        etc. Requires that species are in the same order.
+        :param other:
+        :return:
+        """
+        if not isinstance(other, Structure):
+            return False
+
+        if len(other) != len(self):
+            return False
+        if not all(other.coded_species == self.coded_species):
+            return False
+        if not np.allclose(other.positions, self.positions):
+            return False
+        if not np.allclose(other.cell, self.cell):
+            return False
+
+        return True
+
+
+class StructureSource(object):
+    def __init__(self):
+        pass
+
+    @abstractmethod
+    def get_next_structure(self) -> Structure:
+        raise NotImplementedError
+
+    def write_file(self):
+        raise NotImplementedError
+
+
+class ForceSource(object):
+    def __init__(self):
+        pass
+
+    @abstractmethod
+    def get_next_force(self, *args, **kwargs) -> "np.ndarray":
+        raise NotImplementedError
+
+    def pre_force(self, *args, **kwargs):
+        raise NotImplementedError
+
+
+class Trajectory(StructureSource, ForceSource):
+    def __init__(
+        self, frames: List[Structure] = None, iterate_strategy: Union[int, str] = 1
+    ):
+        if frames is None:
+            frames = []
+        self.frames = frames
+        self.cur_idx = 0
+        self.iterate_strategy = iterate_strategy
+        self.seen_before = []
+
+        if self.iterate_strategy == "shuffle":
+            self.frames = np.random.shuffle(frames)
+        if isinstance(iterate_strategy, int):
+            self.frames = frames[::iterate_strategy]
+
+    def get_next_structure(self) -> Union[Structure, None]:
+
+        if self.cur_idx == len(self):
+            self.cur_idx = 0
+            return None
+
+        cur_frame = self.frames[self.cur_idx]
+        self.cur_idx += 1
+        return cur_frame
+
+    @property
+    def cur_frame(self):
+        return self.frames[self.cur_idx]
+
+    @property
+    def cur_forces(self):
+        return self.frames[self.cur_idx].forces
+
+    def get_next_force(self, index: int = -1) -> "np.ndarray":
+        """
+        Return the forces associated with a current structure,
+        and if an index is passed, that atom.
+        :param index:
+        :return:
+        """
+        if index != 1:
+            return self.cur_frame.forces[index]
+        return self.cur_frame.forces
+
+    def __getitem__(self, item):
+        return self.frames[item]
+
+    def __len__(self):
+        return len(self.frames)
+
+    def __next__(self):
+        struc = self.get_next_structure()
+        if struc is None:
+            raise StopIteration()
+        return struc
+
+    def __iter__(self):
+        self.cur_idx = 0
+        return self
+
+    def append(self, frame: Structure):
+        self.frames.append(frame)
+
 
 def get_unique_species(species: List[Any]) -> (List, List[int]):
     """
